import pandas as pd
from whoosh import index
from whoosh import fields
from whoosh import qparser
from  conf.configuration import *
from whoosh.analysis import StandardAnalyzer
from pathlib import Path
from whoosh.qparser import OrGroup
import logging
from datetime import datetime
import shutil
logging.basicConfig(format="%(message)s",filename="../logs/citing-paper-indexing.log",level=logging.DEBUG)

def create_schema():
    st_ana = StandardAnalyzer()# StandardAnalyzer skips stop words, tokenize by spacy, and lower
    schema = fields.Schema(id=fields.STORED,
                           text=fields.TEXT(analyzer=st_ana, stored=True),
                           )
    return  schema

def index_papers():
    logging.warning(datetime.now())

    corpora = load_corpora_list()
    schema=create_schema()
    for corpus in corpora:
        logging.warning("indexing %s"%corpus)
        path_citing_papers_text = get_path_citing_papers_text(corpus)
        path_corpus_index = get_path_citing_papers_index(corpus)
        if path_corpus_index!=None and path_citing_papers_text!=None:
            df_citing_papers_text = pd.read_csv(path_citing_papers_text,sep=",",encoding="utf-8",quotechar="\"")
            df_citing_papers_text=df_citing_papers_text.dropna(subset=['text'])
            index_path=Path(path_corpus_index)
            ix = index.create_in(index_path, schema)
            writer = ix.writer()
            for i,citing_paper in df_citing_papers_text.iterrows():
                id=citing_paper['id']
                logging.warning("indexing paper %d"%id)
                text=citing_paper['text']
                if text!=None:
                    writer.add_document(id=id,text=text)
            writer.commit()

def find_experiment_papers():

    df_corpora_queries=pd.read_csv("/mnt/ceph/storage/data-in-progress/topic-ontologies/citing-papers/corpora-queries-no-citations.csv",sep=",",encoding="utf-8")
    df_corpora_queries=df_corpora_queries.melt('corpus',value_name='query')

    df_corpora_queries=df_corpora_queries.dropna(subset=['query'])
    df_corpora_queries['query']=df_corpora_queries['query'].astype(str)
    for corpus in load_corpora_list():
        path_corpus_index = get_path_citing_papers_index(corpus)
        path_corpus_papers= get_path_citing_papers(corpus)
        path_experiment_papers=get_path_experiment_papers(corpus)
        print(corpus,path_experiment_papers)
        if path_corpus_papers!=None and path_corpus_index!=None:
            ix = index.open_dir(path_corpus_index)
            with ix.searcher() as searcher:
                corpus_queries=df_corpora_queries[df_corpora_queries['corpus']==corpus]

                for ind, corpus_query in corpus_queries.iterrows():
                    query=corpus_query['query']
                    logging.warning(query)
                    if query!=None:
                        qp = qparser.QueryParser("text", schema=ix.schema,)
                        q= qp.parse(corpus_query['query'])
                        results = searcher.search(q,limit=300)
                        rank =0
                        for res in results:
                            rank=rank+1
                            paper_id= str(res['id'])
                            from_path=path_corpus_papers+"/"+paper_id+".pdf"

                            to_path=path_experiment_papers+"/"+paper_id+".pdf"
                            try:
                                shutil.copy(from_path,to_path)
                            except Exception as error:
                                logging.warning(error)



import sys
sys.path.insert(0,"/home/yamenajjour/git/topic-ontologies/")

from pyspark.sql import SparkSession
import argument_esa_model.esa_all_terms
import argument_esa_model.esa_top_n_terms
from conf.configuration import *
def get_vocab_size(model):
    if '-' in model:
        vocab_size=int(model.split("-")[-1])
    else:
        vocab_size=10
    return vocab_size

def dict_to_list(dictionary):
    vector  = []
    for key in sorted(dictionary):
        vector.append(dictionary[key])
    return vector



spark = SparkSession.builder.appName('topic-ontologies').config('master','yarn').getOrCreate()

spark_context= spark.sparkContext
URI = spark_context._gateway.jvm.java.net.URI
Path = spark_context._gateway.jvm.org.apache.hadoop.fs.Path
FileSystem = spark_context._gateway.jvm.org.apache.hadoop.fs.FileSystem
FsPermission = spark_context._gateway.jvm.org.apache.hadoop.fs.permission.FsPermission
fs = FileSystem.get(URI("hdfs://nn.hdfs.webis.de:8020"), spark_context._jsc.hadoopConfiguration())
fs.listStatus(Path('/user/befi8957'))
log4jLogger = spark_context._gateway.jvm.org.apache.log4j
LOGGER = log4jLogger.LogManager.getLogger(__name__)
set_cluster_mode()
def project_arguments(topic_ontology,model,is_testing):

    vocab_size=get_vocab_size(model)
    def project(argument):
        if is_testing:
            return dict_to_list({"a":0.3,"b":0.3,"c":0.4})
        set_cluster_mode()
        path_word2vec_model = get_path_topic_model('word2vec','word2vec')
        path_word2vec_vocab = get_path_vocab('word2vec')
        if model == 'esa':
            path_topic_model = get_path_topic_model('ontology-'+topic_ontology,model)
            vector = argument_esa_model.esa_all_terms.model_topic(path_topic_model,path_word2vec_model,path_word2vec_vocab,'cos',argument)
            return dict_to_list(vector)
        else:
            import nltk
            nltk.data.path.append('/mnt/ceph/storage/data-in-progress/topic-ontologies/nltk/')
            set_cluster_mode()
            path_corpus = get_path_source("ontology-"+topic_ontology)
            path_word2vec_model = get_path_topic_model('word2vec','word2vec')
            path_word2vec_vocab = get_path_vocab('word2vec')
            vector = argument_esa_model.esa_top_n_terms.model_topic(path_corpus,path_word2vec_model,path_word2vec_vocab,argument,vocab_size)
            return dict_to_list(vector)



    part_count=get_part_count('argument-inventory')
    for corpus in range(0,part_count):
        LOGGER.info("Current corpus:"+str(corpus))
        LOGGER.warn("Current corpus:"+str(corpus))
        path_documents=get_path_preprocessed_documents_version('argument-inventory',str(corpus))
        path_document_vectors = get_path_document_vectors_part('argument-inventory',topic_ontology,model,corpus)
        path_document_vectors_cluster= path_document_vectors .replace(".csv","")
        documents_df  = spark.read.format("csv").option("header", "true").option("delimiter", ",", ).option('quote', '"').load(path_documents).na.drop()
        documents = documents_df.select("document").rdd.map(lambda r: r[0]).repartition(1000)
        ids = (documents_df.select("id").rdd.map(lambda r: r[0])).repartition(1000)
        vectors = documents.map(lambda document:project(document))
        ids_with_vectors=vectors.zip(ids)
        FileSystem.mkdirs(fs,Path(path_document_vectors_cluster),FsPermission(77777))
        ids_with_vectors.saveAsPickleFile(path_document_vectors_cluster)

model = sys.argv[2]
ontology = sys.argv[1]


LOGGER.warn("Lok: processing %s %s"%(model,ontology))
project_arguments(ontology,model,True)


from conf.configuration import *
import pandas as pd
from utils.parts import *
import random
import csv
def log_status(counter,size,debug):
    percentage=(float(counter)/size)*100
    if debug and random.randint(0,1000) %997 ==0:
        logging.warning("finished %2.2f"%percentage)

def produce_suggestions(dataset,ontology,k,debug=False):
    path_top_k_results=get_path_top_k_topics_per_document(dataset,ontology,'random',k)
    path_ontology_topics=get_path_topics('ontology-'+ontology)
    df_ontology_topics=pd.read_csv(path_ontology_topics,sep=',',quotechar='"',quoting=csv.QUOTE_ALL,encoding='utf-8',dtype={'id': object})
    df_ontology_topics.rename(columns={'id':'ontology-topic-id'},inplace=True)
    labels=[]

    for df_part in parts(dataset):
        counter=0
        for index,document in df_part.iterrows():
            size=df_part.shape[0]
            log_status(counter,size,debug)
            relevant_topics=df_ontology_topics['ontology-topic-id'].sample(k).values
            df_relevant_topics=pd.DataFrame({'ontology-topic-id':relevant_topics})
            df_relevant_topics['id']=document['id']
            labels.append(df_relevant_topics)
    df_labels=pd.concat(labels)
    df_labels.to_csv(path_top_k_results,sep=",",quotechar='"',quoting=csv.QUOTE_ALL,encoding="utf-8",index=False)


import os
citing_papers_links_label='citing-papers-links'
citing_papers_text_label='citing-papers-text'
citing_papers_index_label='citing-papers-index'
citing_papers_label='citing-papers'
experiment_papers_label='experiment-papers'
source_label ="source"
sample_label ="sample"
part_count_label='part_count'
judgements_label="judgements"
judgements_model_label="judgements-%s"
evaluation_label="evaluation"
agreement_label="agreement"
ground_truth_corpora_label_template='ground-truth-corpora-%s'
ground_truth_label_template='ground-truth-%s'
ground_truth_raw_label_template='ground-truth-raw-%s'
source_label_template="source-%s"
preprocessed_documents_label = "preprocessed-documents"
preprocessed_topics_label = "preprocessed-topics"
preprocessed_arguments_label = "preprocessed-arguments"
preprocessed_arguments_version_label_template="preprocessed-arguments-%s"
preprocessed_documents_version_label_template= "preprocessed-documents-%s"
preprocessed_part_label_template="preprocessed-%s"
document_topic_label='document-topic'
document_topic_label_template='document-topic-%s'
argument_topic_label='argument-topic'
document_vectors_label_template="document-vectors-%s-%s"
document_vectors_part_label_template="document-vectors-%s-%s-part-%d"
argument_vectors_label_template="argument-vectors-%s-%s"
histogram_label_template= "%s_histogram_%s"
two_parameter_histogram_template= "histogram-%s-over-%s"
two_parameter_histogram_template_figure = "figure-histogram-%s-over-%s"
two_parameter_plot_template_figure = "figure-plot-%s-over-%s"
histogram_label_fig_template= "%s-histogram-%s-fig"
model_label_template="%s-model"
vocab_label_template="%s-vocab"
topics_label="topics"
granularity_label="granularity"
top_k_topics_per_document_label= "top-k-topics-per-document"
top_k_topics_per_argument_label= "top-k-topics-per-argument"
top_topics_per_document_label_threshold= "top-topics-per-document-threshold"
top_topics_per_argument_label_threshold= "top-topics-per-argument-threshold"
all_topics_per_document_label= "all-topics-per-document"
dirname = os.path.dirname(__file__)
cluster_mode=False
cluster_conf_path="/mnt/ceph/storage/data-in-progress/data-research/arguana/topic-ontologies/conf/"
windows_mode=False
hashmap_label_template='hashmap-%s-%s'
labels_model_at_threshold= "labels-model-at-threshold"
labels_model_top_1= "labels-model-top-1"
evaluation_model= "evaluation-model"
judgements_majority="judgements-majority"
documents_label_template="documents-%s"

def get_dataset_conf_path(dataset_name):
    if cluster_mode:
        dataset_conf = cluster_conf_path+("/%s.conf"%dataset_name)
    else:
        dataset_conf = dirname+("/%s.conf"%dataset_name)
    return dataset_conf

def get_path_source(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_source_path = get_property_value(dataset_conf_path,source_label)
    return dataset_source_path

def get_path_ground_truth(dataset_name,ontology):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    ground_truth_label=ground_truth_label_template%ontology
    return get_property_value(dataset_conf_path,ground_truth_label)

def get_path_ground_truth_corpora(dataset_name,ontology):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    ground_truth_corpora_label=ground_truth_corpora_label_template%ontology
    return get_property_value(dataset_conf_path,ground_truth_corpora_label)

def get_path_ground_truth_raw(dataset_name,ontology):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    ground_truth_label=ground_truth_raw_label_template%ontology
    return get_property_value(dataset_conf_path,ground_truth_label)

def get_path_document_topic(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_document_topic=get_property_value(dataset_conf_path,document_topic_label)
    return path_document_topic

def get_path_document_topic_ontology(dataset_name,ontology):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    document_topic_label=document_topic_label_template%ontology
    path_document_topic=get_property_value(dataset_conf_path,document_topic_label)
    return path_document_topic

def get_path_argument_topic(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_argument_topic=get_property_value(dataset_conf_path,argument_topic_label)
    return path_argument_topic

def get_path_judgement(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_judgement= get_property_value(dataset_conf_path,judgements_label)
    return path_judgement

def get_path_judgements_model(dataset_name,model):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    judgements_label=judgements_model_label%model
    return get_property_value(dataset_conf_path,judgements_label)

def get_path_evaluation(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_evaluation= get_property_value(dataset_conf_path,evaluation_label)
    return path_evaluation

def get_path_source_part(dataset_name, subdataset):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    source_label = source_label_template % subdataset
    dataset_source_subset_path = get_property_value(dataset_conf_path,source_label)
    return dataset_source_subset_path

def get_sample_path(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_sample_path = get_property_value(dataset_conf_path,sample_label)
    return dataset_sample_path

def get_path_document_vectors(dataset_name,topic_ontology,topic_model):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    document_vectors_label = document_vectors_label_template % (topic_ontology, topic_model)
    return get_property_value(dataset_conf_path,document_vectors_label)

def get_path_document_vectors_part(dataset_name,topic_ontology,topic_model,part):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    document_vectors_label = document_vectors_part_label_template % (topic_ontology, topic_model,part)
    return get_property_value(dataset_conf_path,document_vectors_label)


def get_path_argument_vectors(dataset_name,topic_ontology,topic_model):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    argument_vectors_label = argument_vectors_label_template % (topic_ontology, topic_model)
    return get_property_value(dataset_conf_path,argument_vectors_label )

def get_path_vocab(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    vocab_label = vocab_label_template% dataset_name
    path_vocab = get_property_value(dataset_conf_path,vocab_label)
    return path_vocab

def get_path_hashmap(dataset_name,source,destination):
    path_dataset_conf=get_dataset_conf_path(dataset_name)
    label=hashmap_label_template%(source,destination)
    path_hashmap=get_property_value(path_dataset_conf,label)
    return path_hashmap

def get_path_preprocessed_documents(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_preprocessed_documents = get_property_value(dataset_conf_path,preprocessed_documents_label)
    return path_preprocessed_documents

def get_path_preprocessed_arguments(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_preprocessed_arguments = get_property_value(dataset_conf_path,preprocessed_arguments_label)
    return path_preprocessed_arguments

def get_path_preprocessed_topics(dataset_name):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_preprocessed_topics = get_property_value(dataset_conf_path,preprocessed_topics_label)
    return path_preprocessed_topics

def get_path_preprocessed_part(dataset_name,part):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    preprocessed_part_label= preprocessed_part_label_template %(part)
    return get_property_value(dataset_conf_path,preprocessed_part_label)

def get_path_preprocessed_arguments_version(dataset_name,version):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    preprocessed_arguments_version_label = preprocessed_arguments_version_label_template% version
    path_preprocessed_arguments_template = get_property_value(dataset_conf_path,preprocessed_arguments_version_label)
    return path_preprocessed_arguments_template

def get_path_preprocessed_documents_version(dataset_name,version):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    path_preprocessed_documents_label = preprocessed_documents_version_label_template%version
    path_preprocessed_documents = get_property_value(dataset_conf_path, path_preprocessed_documents_label)
    return path_preprocessed_documents

def get_histogram_path_figure(dataset_name,attribute):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_histogram_attribute_label = histogram_label_fig_template % (dataset_name, attribute)
    dataset_histogram_figure_attribute_path = get_property_value(dataset_conf_path,dataset_histogram_attribute_label)
    return dataset_histogram_figure_attribute_path

def get_path_histogram_two_parameters(dataset_name, x, y):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_histogram_label = two_parameter_histogram_template % ( x, y)
    dataset_histogram_path = get_property_value(dataset_conf_path,dataset_histogram_label)
    return dataset_histogram_path

def get_path_figure_histogram_two_parameters(dataset_name, x, y):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_histogram_label = two_parameter_histogram_template_figure % (x, y)
    dataset_histogram_figure_path = get_property_value(dataset_conf_path,dataset_histogram_label)
    return dataset_histogram_figure_path

def get_path_figure_plot_two_parameters(dataset_name,x,y):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_plot_label = two_parameter_plot_template_figure% (x, y)
    dataset_histogram_figure_path = get_property_value(dataset_conf_path,dataset_plot_label)
    return dataset_histogram_figure_path

def get_path_topics(ontology_name):
    dataset_conf_path = get_dataset_conf_path(ontology_name)
    path_topics = get_property_value(dataset_conf_path,'topics')
    return path_topics

def get_path_topic_model(ontology_name,model):
    dataset_conf_path = get_dataset_conf_path(ontology_name)
    model_label = model_label_template % model
    path_topic_model = get_property_value(dataset_conf_path,model_label)
    return path_topic_model

def get_histogram_path(dataset_name,attribute):
    dataset_conf_path = get_dataset_conf_path(dataset_name)
    dataset_histogram_attribute_label = histogram_label_template % (dataset_name, attribute)
    dataset_histogram_attribute_path = get_property_value(dataset_conf_path,dataset_histogram_attribute_label)
    return dataset_histogram_attribute_path

def get_path_top_k_topics_per_document(dataset,topic_ontology,topic_model,k):
    dataset_conf_path = get_dataset_conf_path(dataset)
    top_k_topics_per_document_path_template = get_property_value(dataset_conf_path,top_k_topics_per_document_label)
    top_k_topics_per_document_path=  top_k_topics_per_document_path_template % (k,topic_ontology, topic_model)
    return top_k_topics_per_document_path

def get_path_all_topics_per_document(dataset,topic_ontology,topic_model):
    dataset_conf_path = get_dataset_conf_path(dataset)
    all_topics_per_document_path_template = get_property_value(dataset_conf_path,all_topics_per_document_label)
    all_topics_per_document_path=  all_topics_per_document_path_template % (topic_ontology, topic_model)
    return all_topics_per_document_path

def get_path_top_k_topics_per_argument(dataset,topic_ontology,topic_model,k):
    dataset_conf_path = get_dataset_conf_path(dataset)
    top_k_topics_per_argument_path_template = get_property_value(dataset_conf_path,top_k_topics_per_argument_label)
    top_k_topics_per_argument_path=  top_k_topics_per_argument_path_template % (k,topic_ontology, topic_model)
    return top_k_topics_per_argument_path

def get_path_topics_per_document_threshold(dataset,topic_ontology,topic_model,threshold):
    dataset_conf_path = get_dataset_conf_path(dataset)
    top_topics_per_document_path_template = get_property_value(dataset_conf_path,top_topics_per_document_label_threshold)
    path_top_topics_per_document= top_topics_per_document_path_template %(topic_ontology,topic_model,threshold)
    return path_top_topics_per_document

def get_path_topics_per_argument_threshold(dataset,topic_ontology,topic_model,threshold):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_top_topics_per_argument_threshold_template = get_property_value(dataset_conf_path,top_topics_per_argument_label_threshold)
    path_top_topics_per_argument_threshold= path_top_topics_per_argument_threshold_template %(topic_ontology,topic_model,threshold)
    return path_top_topics_per_argument_threshold

def get_path_citing_papers_links(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_citing_papers_links= get_property_value(dataset_conf_path,citing_papers_links_label)
    return path_citing_papers_links

def get_path_citing_papers(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_citing_papers= get_property_value(dataset_conf_path,citing_papers_label)
    return path_citing_papers

def get_path_citing_papers_text(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_citing_papers_text= get_property_value(dataset_conf_path,citing_papers_text_label)
    return path_citing_papers_text

def get_path_citing_papers_index(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_citing_papers_index= get_property_value(dataset_conf_path,citing_papers_index_label)
    return path_citing_papers_index

def get_path_experiment_papers(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_experiment_papers= get_property_value(dataset_conf_path,experiment_papers_label)
    return path_experiment_papers

def get_property_value(dataset_conf_path,property_label):
    conf_file = open(dataset_conf_path,'r')
    for line in conf_file:
        label = line.split("=")[0].strip()
        value = line.split("=")[1].strip()
        root=get_root()
        if label == property_label:
            if value.startswith("/"):
                if value.startswith("/mnt") or value.startswith("/home") :
                    return value
                else:
                    return root+value
            else:
                return value

def get_topic_ontologies():
    return ['strategic-intelligence','strategic-intelligence-sub-topics','wikipedia-categories','wikipedia','debatepedia']

def get_granularity(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    granularity = get_property_value(dataset_conf_path,granularity_label)
    return granularity

def load_corpora_list(corpora_set=None):
    if corpora_set==None:
        corpora_path = get_dataset_conf_path('corpora')
    else:
        corpora_path =get_dataset_conf_path(corpora_set)
    with open(corpora_path,'r') as corpora_file:
        return [l.strip() for l in corpora_file.readlines()]

def get_models():
    return  ['direct-match','esa','esa-lemmatized','word2vec-esa-100','bert','weighted-bert','elmo','weighted-elmo',
              'flair','weighted-flair','glove','weighted-glove']

def get_path_labels_model_at_threshold(dataset,model):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_labels_model_at_threshold=get_property_value(dataset_conf_path,labels_model_at_threshold)
    return path_labels_model_at_threshold%model

def get_path_labels_model_top_1(dataset,model):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_labels_model_top_1_template=get_property_value(dataset_conf_path,labels_model_top_1)
    return path_labels_model_top_1_template%model

def get_path_evaluation_model(dataset,model):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_evaluation_model_template= get_property_value(dataset_conf_path, evaluation_model)
    return path_evaluation_model_template%model

def get_path_judgements_majority(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_judgements= get_property_value(dataset_conf_path,judgements_majority)
    return path_judgements

def get_path_agreement(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    path_agreement= get_property_value(dataset_conf_path,agreement_label)
    return path_agreement


def get_part_count(dataset):
    dataset_conf_path = get_dataset_conf_path(dataset)
    part_count= get_property_value(dataset_conf_path,part_count_label)
    if part_count!=None:
        return int(part_count)
    else:
        return part_count

def get_documents_path(dataset, version):
    path_dataset= get_dataset_conf_path(dataset)
    documents_label = documents_label_template % version
    return get_property_value(path_dataset,documents_label)

def set_cluster_mode():
    global cluster_mode
    cluster_mode=True

def set_windows_mode():
    global windows_mode
    windows_mode=True

def get_root():
    if cluster_mode:
        return "/user/befi8957/topic-ontologies"
    elif windows_mode:
        return "C:\\Users\\user\\disk1\\"
    else:
        return "/mnt/ceph/storage/data-in-progress/data-research/arguana/topic-ontologies"


from conf.configuration import *
import pandas as pd
import urllib.request
import logging
import time
import random
import os.path
from datetime import datetime
import pdftotext
import re
def filter_downloadable():
    corpora=['ukp-argument-annotated-essays-v2']
    #corpora = load_corpora_list()
    websites={
              'arxiv':'https://arxiv.org',
              'acm':'https://dl.acm.org',
              'acl':'https://www.aclweb.org'
              }

    patterns={
        'arxiv':lambda url:url.replace('abs','pdf'),
        'acm':lambda url: url.split('?')[0]+'.pdf' if '?' in url  else (url.replace("abs","pdf") if "abs" in url else url+".pdf"),
        'acl':lambda url: url[:-1]+".pdf"
    }

    for corpus in corpora:
        print(corpus)
        path_citing_papers_links=get_path_citing_papers_links(corpus)
        if path_citing_papers_links!=None:
            df_citing_papers_links = pd.read_csv(path_citing_papers_links,sep=",",encoding="utf-8",quotechar="\"")
            df_citing_papers_links['ArticleURL']=df_citing_papers_links['ArticleURL'].astype(str)
            df_citing_papers_links['pdf']=df_citing_papers_links.apply(lambda row: row['ArticleURL'].endswith('pdf'),axis=1)
            df_citing_papers_links['url']=""
            df_non_pdf_files=df_citing_papers_links[~df_citing_papers_links.pdf]
            for website in websites:
                head=websites[website]
                df_citing_papers_links[website]=df_non_pdf_files.apply(lambda row: row['ArticleURL'].startswith(head),axis=1)

            for index, row in df_citing_papers_links.iterrows():
                if row.pdf:
                    df_citing_papers_links.loc[index,'url']=row['ArticleURL']
                else:
                    for website in websites:
                        pattern=patterns[website]
                        if row[website]:
                            df_citing_papers_links.loc[index,'url']=pattern(row['ArticleURL'])
            df_citing_papers_links['id']=range(0,df_citing_papers_links.shape[0])
            df_citing_papers_links.to_csv(path_citing_papers_links,sep=",",encoding="utf-8",quotechar="\"",index=False)

def setup_mozilla():
    opener = urllib.request.build_opener()
    opener.addheaders = [('User-agent', 'Mozilla/5.0')]
    urllib.request.install_opener(opener)

def download_files(check_existence=False):
    logging.basicConfig(format="%(message)s",filename="../logs/download-citing-papers.log")
    corpora=['ukp-argument-annotated-essays-v2']
    logging.warning(datetime.now())
    #corpora = load_corpora_list()
    #corpora=['upotsdam-arg-microtexts-v2']
    for corpus in corpora:
        logging.warning("download citing papers of %s"%corpus)
        path_citing_papers_links=get_path_citing_papers_links(corpus)
        path_citing_papers=get_path_citing_papers(corpus)
        if path_citing_papers_links!=None:
            df_citing_papers_links = pd.read_csv(path_citing_papers_links,sep=",",encoding="utf-8",quotechar="\"")
            df_citing_papers_links=df_citing_papers_links.loc[df_citing_papers_links['url'].dropna().index]
            for index,paper in df_citing_papers_links.iterrows():
                id=paper['id']
                url=paper['url']

                path=path_citing_papers+"/"+str(id)+".pdf"
                #print(url,path)
                try:
                    setup_mozilla()
                    if check_existence:
                        if not os.path.exists(path):
                            logging.warning("downloading %s"%url)
                            urllib.request.urlretrieve(url,path)
                    else:
                            logging.warning("downloading %s"%url)
                            urllib.request.urlretrieve(url,path)

                except Exception as error:
                    logging.warning(error)
                    logging.warning("could not download %s" %url)

                time.sleep(random.randint(1,20))

def parse_files():
    #logging.basicConfig(format="%(message)s",filename="../logs/parsing-papers.log")
    corpora=load_corpora_list()
    for corpus in corpora:
        logging.warning("parsing papers of %s"%corpus)
        path_citing_papers_links=get_path_citing_papers_links(corpus)
        path_citing_papers_texts=get_path_citing_papers_text(corpus)
        if path_citing_papers_links!=None and os.path.exists(path_citing_papers_links):
            path_citing_papers=get_path_citing_papers(corpus)
            df_citing_papers=pd.read_csv(path_citing_papers_links,sep=",",encoding="utf-8",quotechar="\"")
            df_citing_papers['text']=""
            for index,citing_paper in df_citing_papers.iterrows():
                paper_id=citing_paper['id']
                path_citing_paper = path_citing_papers+"/"+str(paper_id)+".pdf"
                if os.path.exists(path_citing_paper):
                    logging.warning("found %s %s"%(corpus,paper_id))
                    text=""
                    try:
                        with open(path_citing_paper,'rb') as f:
                            paper_pdf = pdftotext.PDF(f)
                            for page in paper_pdf:
                                text=text+page
                    except Exception as error:
                        logging.warning(error)
                    text=text.replace("\t"," ").replace("\n"," ")
                    text=re.sub(r"\s+"," ",text)
                    df_citing_papers.loc[index,'text']=text
            df_citing_papers=df_citing_papers[['id','Abstract','Cites','Title','text']]
            df_citing_papers.to_csv(path_citing_papers_texts,sep=",",encoding="utf-8",quotechar="\"")



import os
from conf.configuration import *


"documents-vectors-wikipedia-esa-ibm-debater-claim-sentence-search-part-5.csv"

def generate_document_parts(dataset,label_template,path_template):
    count=get_part_count(dataset)
    path_dataset=get_dataset_conf_path(dataset)

    with open(path_dataset,'a+') as file:

        for i in range(0,count):
            label = label_template %(i)
            line =label+ "=" + path_template%(i)+"\n"
            file.write(line)

def generate_document_vectors_paths(dataset,label_template,path_vectors_template):
    count=get_part_count(dataset)
    ontologies=get_topic_ontologies()
    path_dataset=get_dataset_conf_path(dataset)
    with open(path_dataset,'a+') as file:
        for ontology in ontologies:
            for i in range(0,count):
                label = label_template %(ontology,'esa',i)
                line =label+ "=" + path_vectors_template%(ontology,'esa',i)+"\n"
                file.write(line)
